*
*  X3CNVCHR.PRG
*  Converts the passed character string to a value
*    of the passed data type.  Returns .NULL. if 
*    unable to process.
*  -OR-
*  Converts the passed value to a string.  Returns
*    .NULL. if unable to process.
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  20110
*                      703-369-1759
*  Author:  Drew Speedie
*
*  lParameters
*       tuValue (R) Value to be converted 
*    tcDataType (O) Data type to convert the tuValue to:
*                     "C"haracter
*                     "N"umeric
*                     "D"ate
*                     Date"T"ime
*                     "L"ogical
*                     "E"xpression
*                   IF tuValue is a character string to
*                     be converted to a specific data
*                     type, tcDataType is required.  
*                   If tcDataType is not passed, this 
*                     routine converts tuValue to a 
*                     character string. 
*
lparameters tuValue, tcDataType
local luReturnValue, lcString, lcDataType, llStringToValue
luReturnValue = .NULL.
IF type("tcDataType") = "C" ;
     AND !isnull(tcDataType) ;
     AND !empty(tcDataType) ;
     AND lenc(alltrim(tcDataType)) = 1 ;
     AND inlist(upper(tcDataType),"C","N","D","L","T","E")
  *
  *  tcDataType was passed as a valid data type
  *
  *  convert character string tuValue to 
  *  the passed data type 
  *
  lcDataType = upper(alltrim(leftc(tcDataType,1)))
  IF lcDataType = "C"
    llStringToValue = .f.
   ELSE
    llStringToValue = .t.
    lcString = tuValue
   ENDIF
 ELSE
  *
  *  tuValue is to be converted to a string
  *  (2nd parameter not passed)
  *
  llStringToValue = .f.
ENDIF
IF llStringToValue
  *
  *  convert the passed string to a value of the
  *  indicated data type
  *
  DO CASE
    **********************************************
    CASE lcDataType = "C"   &&& character
    **********************************************
      luReturnValue = alltrim(lcString)
    **********************************************
    CASE lcDataType = "N"   &&& numeric
    **********************************************
      IF empty(lcString)
        luReturnValue = 0
       ELSE
        lcString = alltrim(lcString)
        local lnDecimals
        lnDecimals = 0
        IF "." $ lcString
          lnDecimals = substrc(lcString,at_c(".",lcString)+1)
          lnDecimals = lenc(lnDecimals)
        ENDIF
        IF lnDecimals > 0
          local lnSetDecimals
          lnSetDecimals = set("DECIMALS")
          set decimals to lnDecimals
          luReturnValue = val(lcString)
          set decimals to lnSetDecimals
         ELSE
          luReturnValue = int(val(lcString))
        ENDIF   
      ENDIF empty(lcString)
    **********************************************
    CASE lcDataType = "D"   &&& date
    **********************************************
      IF empty(lcString) ;
           OR empty(chrtranc(lcString,"/",space(0)))
        luReturnValue = {}
       ELSE
        luReturnValue = ctod(alltrim(lcString))
      ENDIF empty(lcString)
    **********************************************
    CASE lcDataType = "T"   &&& datetime
    **********************************************
      lcString = alltrim(lcString)
      IF empty(lcString)
        luReturnValue = ctot("{}")
       ELSE
        luReturnValue = ctot(lcString)
      ENDIF empty(lcString)
    **********************************************
    CASE lcDataType = "L"   &&& logical
    **********************************************
      lcString = upper(alltrim(lcString))
      DO CASE
        CASE empty(lcString)
          * unable to process
        CASE inlist(lcString,"T",".T.","TRUE","Y","YES")
          luReturnValue = .t.
        CASE inlist(lcString,"F",".F.","FALSE","N","NO")
          luReturnValue = .f.
        OTHERWISE
          * unable to process
      ENDCASE  
    **********************************************
    CASE lcDataType = "E"   &&& expression
    **********************************************
      IF empty(lcString)
       ELSE
        local lcOnError, llError
        lcOnError = on("ERROR")
        llError = .f.
        on error llError = .t.
        luReturnValue = eval(alltrim(lcString))
        on error &lcOnError  
        IF llError
          luReturnValue = .NULL.
        ENDIF
      ENDIF empty(lcString)
  ENDCASE
 ELSE 
  *
  *  convert the passed value to a string
  *
  DO CASE
    *********************************************
    CASE isnull(tuValue) 
    *********************************************
      luReturnValue = ".NULL."
    *********************************************
    CASE type("tuValue") = "C"
    *********************************************
      luReturnValue = alltrim(tuValue)
    *********************************************
    CASE type("tuValue") = "N"
    *********************************************
      luReturnValue = alltrim( padr( tuValue, 254 ) )
    *********************************************
    CASE type("tuValue") = "D"
    *********************************************
       luReturnValue = dtoc(tuValue)
    *********************************************
    CASE type("tuValue") = "T"
    *********************************************
       luReturnValue = ttoc(tuValue)
    *********************************************
    CASE type("tuValue") = "L"
    *********************************************
       luReturnValue = IIF(tuValue,".T.",".F.")
  ENDCASE
ENDIF
return luReturnValue  