*
*  X3ERROR.PRG
*  Global error handler 
*
*  This error handler is intended to be invoked using
*  standard ON ERROR do x3error... syntax, but it can
*  also be explicitly called from the Error() method
*  of individual forms/controls/objects.
*
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  20110
*                      703-369-1759
*  Author:  Drew Speedie
*
*     
*          Yes, I realize localization hasn't been accounted
*          for yet...
*
*
*  REQUIRES
*    ERRORLOG.DBF must be open (in the default data session)
*    and in the VFP path.
*
*
*  SEE ALSO
*    X3FW.VCX/ctrApp.InstallGlobalErrorHandler()
*
*
*  USAGE
*    on error do x3error with error(), ;
*                             program(), ;
*                             lineno(), ;
*                             sys(16)
*
*
*  lParameters
*    tnErrorNumber (R) Number of the error (corresponds
*                        to the nError parameter of the
*                        Error() event method)
*    tcMethodOrPRG (R) Method/Program where the error occurred
*                        (corresponds to the cMethod parameter
*                        of the Error() event method)
*           tnLine (R) Line of code on which the error occurred
*                        (corresponds to the nLine parameter
*                        of the Error() event method)
*          tcSYS16 (R) SYS(16) when the error occurred
*         toObject (O) Object reference to the THIS object when
*                        the error occurred
*
lparameters tnErrorNumber, ;
            tcMethodOrPRG, ;
            tnLine, ;
            tcSYS16, ;
            toObject
private pcMessage
pcMessage = message()

IF type("gnErrorNumber") # "N"
  public gnErrorNumber
ENDIF
gnErrorNumber = tnErrorNumber

DO CASE
  ******************************************************************
  CASE "," + alltrim(str(gnErrorNumber)) + "," $ ",125," 
  ******************************************************************
    IF messagebox("The printer is not ready.", ;
                  5+48, ;
                  "Printer error") = 4
      retry
    ENDIF
    lcAction = "NOTHING"                  
  ******************************************************************
  CASE "," + alltrim(str(gnErrorNumber)) + "," $ ;
       ",3,108,109,110,130,1502,1503,1705,1708,1101,1106,1637,"
  ******************************************************************
    *
    *  multi-user errors
    *
    lcAction = "NOTHING"
  ******************************************************************
  CASE (val(sys(1001))-val(sys(1016))) < 10000 ;
       OR "," + alltrim(str(gnErrorNumber)) + "," $ ;
       ",6,56,1410,43,1809,1149,1150,1151,1600" 
  ******************************************************************
    *
    *  low memory or not enough file handles, etc.,
    *  so we're not liable to be able to do anything more
    *  than attempt to write to the error log and 
    *  get the heck out of here
    *
    lcAction = "EMERGENCY"
  ******************************************************************
  OTHERWISE
  ******************************************************************
    *
    *  all other errors
    *
    lcAction = "SHUTDOWN"
ENDCASE

IF lcAction = "NOTHING"
  *
  *  multi-user errors don't cause a system shutdown,
  *  but gnErrorNumber now contains the error number
  *  and can be queried like this:
  *
  *    select SOMETABLE
  *    gnErrorNumber = 0
  *    append blank
  *    IF gnErrorNumber > 0
  *      *  the APPEND BLANK failed
  *     ELSE
  *      *  whatever
  *    ENDIF
  *
  return
ENDIF  

wait window "An unrecoverable system error has occurred.  " + chr(13) + ;
            "Please wait while this error is logged..." nowait

IF lcAction = "EMERGENCY"
  *
  *  Log just the bare essentials.  This was the least
  *  memory/disk-intensive way I could think of to do
  *  this.  Creating a .TXT file and other options 
  *  probably require more resources than simply 
  *  attempting to write a record to ERRORLOG, which
  *  should already be open at this point
  *
  set datasession to 1
  insert into ERRORLOG (er_dattime, ;
                        er_user, ;
                        er_error, ;
                        er_methprg, ;
                        er_lineno, ;
                        er_program, ;
                        er_code) ;
                values (datetime(), ;
                        IIF(type("oUser")="O" AND !isnull("oUser"), oUser.GetPProp("icUsr_PK"), "?????"), ;
                        alltrim(str(tnErrorNumber)), ;
                        tcMethodOrPRG, ;
                        alltrim(str(tnLine)), ;
                        tcSYS16,; 
                        "(unlogged)")
  wait window "An unrecoverable system error has occurred.  " + chr(13) + ;
              "Please report this problem.  " + chr(13) + ;
              "Press any key to terminate this program..." ;
              to junk timeout 300
  on shutdown
  _Screen.Visible = .t.
  quit
ENDIF

*  lcAction = "SHUTDOWN"

?? chr(7)

*
*  attempt to log the error
*
private pcErrorDBF
pcErrorDBF = space(0)
do LogError with tnErrorNumber, ;
                 tcMethodOrPRG, ;
                 tnLine, ;
                 tcSYS16, ;
                 toObject

*
*  prevent any AlwaysOnTop forms from getting in
*  the way
*
IF type("_Screen.ActiveForm") = "O" AND !isnull(_Screen.ActiveForm)
  local xx
  FOR xx = 1 to _Screen.FormCount
    IF type("_Screen.Forms[xx].AlwaysOnTop") = "L" ;
         AND _Screen.Forms[xx].AlwaysOnTop = .t. 
      _Screen.Forms[xx].AlwaysOnTop = .f.
    ENDIF
  ENDFOR loop thru all forms
ENDIF

do ShowError
*
*  now that we've done things in an orderly fashion,
*  update the flag in ERRORLOG that tells us if the
*  user re-booted (or something else bad happened)
*  on the way out -- if ERRORLOG.er_reboot = .t.,
*  it means we never made it this far)
*             
replace er_reboot with .f. in (pcErrorDBF)
*
*  issuing the QUIT command fires whatever is the
*  current ON SHUTDOWN, so we have to clear that out
*  first
*
on shutdown
*
*  if the current form is modal, we need to explicitly
*  release it here to be absolutely sure it doesn't
*  intefere with the shutdown process
*
IF type("_Screen.ActiveForm") = "O" ;
     AND !isnull(_Screen.ActiveForm) ;
     AND ( (type("_Screen.ActiveForm.WindowType") = "N" AND _Screen.ActiveForm.WindowType = 1) ;
          OR ;
           (type("_Screen.ActiveForm.ilVM") # "U" AND _Screen.ActiveForm.GetPProp("WindowType") = 1) )
  _Screen.ActiveForm.Release()
ENDIF
*
*  hit the road
*
IF type("oApp") = "O" AND !isnull(oApp) AND oApp.InDevelopment()
  _Screen.Visible = .t.   &&& in case we're in VFP5
  IF messagebox("You can either CANCEL or QUIT now.  (CANCEL returns you to the Command Window)" + chr(13) + chr(13) + ;
                "Select <Cancel> to CANCEL, <OK> to QUIT", ;
                1+48+256, ;
                "CANCEL or QUIT") = 1
    quit
   ELSE
    cancel
  ENDIF                
 ELSE
  *
  *  oApp doesn't exist or oApp.InProduction()
  *
  quit
ENDIF
return            
      
      
      

PROCEDURE LogError
*
*  log information about the VFP app environment
*  at the time of the error
*
lparameters tnErrorNumber, ;
            tcMethodOrPRG, ;
            tnLine, ;
            tcSYS16, ;
            toObject
local array laError[1]
=aerror(laError)
*
*  set up some stuff 
*
local lcTextFile, lcSaveDBC, lcSaveAlias
lcSaveDBC   = set("DATABASE")
lcSaveAlias = alias()
lcTextFile  = sys(3) + ".TXT"
DO WHILE file(lcTextFile)
  lcTextFile = sys(3) + ".TXT"
ENDDO
*
*  this is a little tricky.  We can't just SELECT ERRORLOG
*  because we're not likely to be in the default data 
*  session, in which case it's not USED()
* 
pcErrorDBF = "ERRORLOG"
IF !used(pcErrorDBF)
  use (pcErrorDBF) in 0 again alias X3ERROR shared
  pcErrorDBF = "X3ERROR"
ENDIF
*
*  log the initial error stuff
*
select (pcErrorDBF)
append blank
replace er_dattime with datetime() 
replace er_user    with IIF(type("oUser")="O" AND !isnull(oUser), oUser.GetPProp("icUsr_PK"), "?????")
replace er_reboot  with .t.
replace er_error   with alltrim(str(laError[1]))
replace er_methprg with rightc(tcMethodOrPRG,len(er_methprg))
replace er_lineno  with alltrim(str(tnLine))
replace er_message with pcMessage  &&& laError[2]
replace er_code    with message(1) 

private pcCRLF 
pcCRLF = chr(13) + chr(10)

*
*  log the AERROR() contents
*
local xx 
FOR xx = 1 to alen(laError,1)
  replace er_aerror with "AERROR() contents" + pcCRLF
  replace er_aerror with er_aerror + alltrim(str(laError[xx,1])) + pcCRLF
  replace er_aerror with er_aerror + laError[xx,2] + pcCRLF
  replace er_aerror with er_aerror + nvl(laError[xx,3],".NULL.") + pcCRLF
  DO CASE
    CASE isnull(laError[xx,4])
      replace er_aerror with er_aerror + ".NULL." + pcCRLF
    CASE type("laError[xx,4]") = "C"
      replace er_aerror with er_aerror + laError[xx,4] + pcCRLF
    CASE type("laError[xx,4]") = "N"
      replace er_aerror with er_aerror + alltrim(str(laError[xx,4])) + pcCRLF
    OTHERWISE                       
      replace er_aerror with er_aerror + "column 4 is unknown" + pcCRLF
  ENDCASE
  DO CASE
    CASE isnull(laError[xx,5])
      replace er_aerror with er_aerror + ".NULL." + pcCRLF
    CASE type("laError[xx,5]") = "C"
      replace er_aerror with er_aerror + laError[xx,5] + pcCRLF
    CASE type("laError[xx,5]") = "N"
      replace er_aerror with er_aerror + alltrim(str(laError[xx,5])) + pcCRLF
    OTHERWISE                       
      replace er_aerror with er_aerror + "column 5 is unknown" + pcCRLF
  ENDCASE
  replace er_aerror with er_aerror + nvl(laError[xx,6],".NULL.") + pcCRLF
  IF isnull(laError[xx,7])
    replace er_aerror with er_aerror + ".NULL." + pcCRLF
   ELSE
    replace er_aerror with er_aerror + alltrim(str(laError[xx,7]))
  ENDIF 
ENDFOR
*
*  log the program execution path
*
local yy
xx = 1
yy = 1
DO WHILE !empty(sys(16,xx)) 
  IF sys(16,xx) # "ON." .and. !"X3ERROR"$sys(16,xx)
    replace er_program with er_program + sys(16,xx) + pcCRLF in (pcErrorDBF)
  ENDIF
  xx = xx + 1
ENDDO 
replace er_program with er_program + "(" + tcMethodOrPrg + ")"
release xx
release yy
*
*  log the current active form
*
IF type("_Screen.ActiveForm") = "O" AND !isnull(_Screen.ActiveForm)
  replace er_forms with "CURRENT ACTIVEFORM" + pcCRLF in (pcErrorDBF)
  do er_forms_loop with _Screen.ActiveForm  
 ELSE
  replace er_forms with "no current active forms -- Default Data Session:" + pcCRLF in (pcErrorDBF)
  do er_forms_loop with _Screen 
ENDIF
*
*  log memory
*
list memory to file (lcTextFile) noconsole
select (pcErrorDBF)
append memo er_lstmemo from (lcTextFile)
*
*  log status
*
list status to file (lcTextFile) noconsole
select (pcErrorDBF)
append memo er_lststat from (lcTextFile)

erase (lcTextFile)

*
*  log any CONFIG.FPW in effect
*
select (pcErrorDBF)
IF !empty(sys(2019)) AND file(sys(2019))
  append memo er_config from (sys(2019))
  replace er_config with sys(2019) + pcCRLF + er_config
 ELSE
  replace er_config with "no CONFIG.FPW file"
ENDIF
*
*  log other stuff in the environment
*
replace er_environ with              "             Disk remaining: " + alltrim(str(diskspace())) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                  Disk size: " + sys(2020) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "Available memory below 640K: " + sys(12) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "           Available memory: " + sys(1001) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "              Memory in use: " + sys(1016) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                        EMS: " + sys(23) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                  EMS limit: " + sys(24) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "   Foreground memory buffer: " + sys(3050,1) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "   Background memory buffer: " + sys(3050,2) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "          Current directory: " + sys(5)+sys(2003) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "           FoxPro start dir: " + home() + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "              Resource file: " + sys(2005) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "       Temporary file drive: " + sys(2023) + pcCRLF  in (pcErrorDBF)
replace er_environ with er_environ + "              Graphics card: " + sys(2006) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "        Files in Config.SYS: " + sys(2010) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                    Printer: " + sys(6) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "             Printer status: " + sys(13) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                    Version: " + version(1) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                    Machine: " + sys(0) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "           Operating System: " + os(1) + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "Underlying Operating System: " + os() + pcCRLF in (pcErrorDBF)
replace er_environ with er_environ + "                  Processor: " + sys(17) + pcCRLF in (pcErrorDBF)


*
*  log each extant form
*
select (pcErrorDBF)
IF _Screen.FormCount > 0
  replace er_forms with er_forms + "EXTANT FORMS" + pcCRLF in (pcErrorDBF)
  local xx
  FOR xx = 1 to _Screen.FormCount
    do er_forms_loop with _Screen.Forms[xx]
  ENDFOR
 ELSE
  replace er_forms with er_forms + "No extant forms" in (pcErrorDBF)
ENDIF

IF !empty(lcSaveAlias)
  select (lcSaveAlias)
ENDIF
IF !empty(lcSaveDBC)
  set database to (lcSaveDBC)
ENDIF

return            



PROCEDURE ShowError
*
*  Present the user with a dialog describing the
*  error, etc.
*
?? chr(7)
select (pcErrorDBF)
local loForm
IF VFPVersion() = 3
  loForm = createobject("frmShowError3")
 ELSE
  loForm = createobject("frmShowError5")
ENDIF
loForm.Show()
return



PROCEDURE er_forms_loop
*
*  gather and store ERRORLOG.Er_Forms data for the
*  passed form
*
lParameters toForm
local lnDataSession, lnOffset
lnDataSession = set("DATASESSION")
lnOffset = 2
replace er_forms with er_forms + space(lnOffset) + "             Name  " + toForm.Name + pcCRLF in (pcErrorDBF)
local lcSCX
lcSCX = sys(1271,toForm)
IF type("lcSCX") = "L"
  lcSCX = "not an .SCX-based form"
ENDIF
replace er_forms with er_forms + space(lnOffset) + "        .SCX file  " + lcSCX + pcCRLF in (pcErrorDBF)
replace er_forms with er_forms + space(lnOffset) + "          Caption  " + toForm.Caption + pcCRLF in (pcErrorDBF)
replace er_forms with er_forms + space(lnOffset) + "            Class  " + toForm.Class + pcCRLF in (pcErrorDBF)
replace er_forms with er_forms + space(lnOffset) + "     ClassLibrary  " + toForm.ClassLibrary + pcCRLF in (pcErrorDBF)
replace er_forms with er_forms + space(lnOffset) + "      DataSession  " + IIF(toForm.DataSession=1,"Default","Private") + pcCRLF in (pcErrorDBF)
replace er_forms with er_forms + space(lnOffset) + "    DataSessionID  " + alltrim(str(toForm.DataSessionID)) + pcCRLF in (pcErrorDBF)
set datasession to toForm.DataSessionID
local lcString
lcString = space(0)
lcString = lcString + space(lnOffset) + "Transaction level  " + alltrim(str(txnlevel())) + pcCRLF
lcString = lcString + space(lnOffset) + "  SET DATABASE TO  " + IIF(empty(set("DATABASE")),"none",set("DATABASE")) + pcCRLF 
lcString = lcString + space(lnOffset) + "    Current DBC()  " + IIF(empty(set("DATABASE")),"none",dbc()) + pcCRLF
lcString = lcString + space(lnOffset) + "       WORK AREAS  " + pcCRLF 
local array laCursors[1]
local lnCursors
lnCursors = aused(laCursors)
IF lnCursors > 0
  local lcAlias, xx
  FOR xx = 1 to lnCursors
    lcAlias = laCursors[xx,1]
    select (lcAlias)
    lcString = lcString + space(lnOffset*6) + lcAlias + pcCRLF
    lcString = lcString + space(lnOffset*4) + "             DBF()  " + dbf(lcAlias) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "           RECNO()  " + IIF(eof(lcAlias),"eof()",alltrim(str(recno(lcAlias)))) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "             EOF()  " + IIF(eof(lcAlias),"yes","no") + pcCRLF
    lcString = lcString + space(lnOffset*4) + "           ORDER()  " + IIF(empty(order(lcAlias)),"none",order(lcAlias)) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "          FILTER()  " + IIF(empty(filter(lcAlias)),"none",filter(lcAlias)) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "          DATABASE  " + IIF(empty(cursorgetprop("DATABASE",lcAlias)),"free table",cursorgetprop("DATABASE",lcAlias)) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "        SourceType  " + alltrim(str(cursorgetprop("SourceType",lcAlias))) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "        SourceName  " + cursorgetprop("SourceName",lcAlias) + pcCRLF
    lcString = lcString + space(lnOffset*4) + "        BufferMode  " + alltrim(str(cursorgetprop("BUFFERING",lcAlias))) + pcCRLF
    if .f.
    *
    *  I had to suppress this one because GetNextModified() has
    *  an implicit record pointer move that fires the field/row
    *  rule(s), possibly generating another error here
    *
    IF cursorgetprop("BUFFERING") > 3
    lcString = lcString + space(lnOffset*4) + "GetNextModified(0)  " + alltrim(str(getnextmodified(0,lcAlias))) + pcCRLF
    ENDIF
    endif
    IF cursorgetprop("BUFFERING") > 1
    IF eof()
    lcString = lcString + space(lnOffset*4) + "   GetFldState(-1)  " + "eof()" + pcCRLF
    ELSE
    lcString = lcString + space(lnOffset*4) + "   GetFldState(-1)  " + getfldstate(-1,lcAlias) + pcCRLF
    ENDIF
    ENDIF    
  ENDFOR log all work areas
ENDIF there are cursors open in this data session
*
*  we can't QUIT while in the middle of a transaction, 
*  nor should we leave any dirty buffers
*
*  rollback any extant transactions
*
DO WHILE txnlevel() > 0
  rollback
ENDDO
*
*  revert any dirty buffers
*
IF lnCursors > 0
  FOR xx = 1 to lnCursors
    IF cursorgetprop("BUFFERING",laCursors[xx,1])>1
      select (laCursors[xx,1])
      =tablerevert(.t.)
    ENDIF
  ENDFOR
ENDIF any cursors open in this data session
set datasession to lnDataSession
select (pcErrorDBF)
replace er_forms with er_forms + lcString in (pcErrorDBF)
return



DEFINE CLASS frmShowError3 AS Form
  *
  *  the form to display to the user is defined here
  *  so that we don't have to rely on any external
  *  files, which we cannot assume are available at
  *  this point
  *

  AutoCenter = .t.
  Height = 372
  Width = 560
  BackColor = RGB(192,192,192)
  BorderStyle = 2
  Closable = .F.
  MaxButton = .F.
  MinButton = .F.
  WindowType = 1
  Name = "frmShowError"
  FontName = "Arial"
  FontSize = 9
  FontBold = .f.

  ADD OBJECT label1 AS lblError WITH ;
	Height = 33, ;
	Top = 16, ;
	Width = 521, ;
	WordWrap = .T., ;
	Name = "Label1"

  ADD OBJECT label2 AS lblErrorPrompt WITH ;
	Top = 52, ;
	Name = "Label2", ;
	Alignment = 1

  ADD OBJECT label3 AS lblErrorPrompt WITH ;
	Top = 84, ;
	Name = "Label3", ;
	Alignment = 1

  ADD OBJECT label4 AS lblErrorPrompt WITH ;
	Top = 116, ;
	Name = "Label4", ;
	Alignment = 1

  ADD OBJECT label5 AS lblErrorInfo WITH ;
	Top = 52, ;
	Name = "Label5"

  ADD OBJECT label6 AS lblErrorInfo WITH ;
	Top = 84, ;
	Name = "Label6"

  ADD OBJECT label7 AS lblErrorInfo WITH ;
	Height = 51, ;
	Top = 116, ;
	Name = "Label7"

  ADD OBJECT label8 AS lblError WITH ;
	Height = 51, ;
	Top = 172, ;
	Width = 521, ;
	WordWrap = .T., ;
	Name = "Label8"

  ADD OBJECT edit1 AS editbox WITH ;
	FontBold = .F., ;
	FontSize = 9, ;
	Height = 81, ;
	Left = 20, ;
	Top = 236, ;
	Width = 521, ;
	Name = "Edit1"

  ADD OBJECT label9 AS lblError WITH ;
	AutoSize = .T., ;
	Left = 20, ;
	Top = 320, ;
	Name = "Label9"

  ADD OBJECT command1 AS commandbutton WITH ;
	Top = 332, ;
	Left = 472, ;
	Height = 27, ;
	Width = 69, ;
	FontBold = .F., ;
	FontSize = 9, ;
	Name = "Command1"

  *
  *  while we'd like the user to see this form,
  *  and maybe enter some useful information,
  *  an error might happen while they're away
  *  from their workstation, so we'll timeout
  *  after an hour to make sure cleanup, release
  *  of any RLOCK()s, etc. takes place
  *
  ADD OBJECT timer1 AS timer WITH ;
	Top = 0, ;
	Left = 0, ;
	Height = 23, ;
	Width = 23, ;
	Interval = 3600000, ;
	Name = "Timer1"

  PROCEDURE Init
    *
    *  all strings are built here to make it easier
    *  to localize them in the future
    *
    select (pcErrorDBF)
    WITH THIS
      .Caption = "System Error"
      .Label1.Caption = "A system error has occurred; please report this problem.  Complete error has been logged, but here is the important information:"
      .Label2.Caption = "Error"
      .Label3.Caption = "Program section"
      .Label4.Caption = "Problem code"
      .Label5.Caption = alltrim(er_error) + "/" + alltrim(er_message)
      local xx 
      xx = substr(er_program,ratc(chr(13)+chr(10),er_program)+3)
      xx = leftc(xx,len(xx)-1)
      .Label6.Caption = xx
      .Label7.Caption = alltrim(er_code)
      .Label8.Caption = "We'd like to prevent this from happening again.  Please take a few moments to describe what you were doing when this error occurred, what activities you were doing just before this happened, and anything else you can think of that might be of help."
      .Label9.Caption = "Thanks very much"
      local loLabel
      FOR xx = 1 to 9
        loLabel = ".Label" + alltrim(str(xx))
        loLabel = &loLabel
        IF loLabel.Width # 521
          loLabel.Width = THIS.TextWidth(loLabel.Caption)
        ENDIF  
        IF inlist(xx,2,3,4)
          loLabel.Left = .Label5.Left - 4 - loLabel.Width
        ENDIF
      ENDFOR

      .Edit1.ControlSource = "er_usinfo"
      .Command1.Caption = "\<OK"
    ENDWITH
  ENDPROC

  PROCEDURE Command1.Click
    THISFORM.Release()
  ENDPROC

  PROCEDURE Timer1.Timer
    THISFORM.Release()
  ENDPROC

ENDDEFINE


DEFINE CLASS lblError AS Label
  Caption = space(0)
  BackStyle = 0
  FontName = "Arial"
  FontBold = .F.
  FontSize = 9
  Height = 17
  Left = 20
ENDDEFINE


DEFINE CLASS lblErrorPrompt AS lblError
  Alignment = 1
  Left = 24
  Width = 200
ENDDEFINE


DEFINE CLASS lblErrorInfo AS lblError
  ForeColor = RGB(255,0,0) 
  Height = 33 
  Left = 232
  Width = 309 
  WordWrap = .T.
ENDDEFINE




DEFINE CLASS frmShowError5 AS frmShowError3
  ShowWindow = 1
ENDDEFINE



PROCEDURE VFPVersion
*
*  Yes, this is identical to X3GETVER.PRG, but I've
*  included it here as a local procedure since we're
*  in error mode, and I don't want to take for granted
*  that X3GETVER.PRG is available
*
*
local lcString, lnVersion, xx
lcVersion = upper(alltrim(version()))
lcVersion = strtran(lcVersion,"VISUAL FOXPRO",space(0))
lcString  = space(0)
FOR xx = 1 to lenc(lcVersion)
  IF empty(chrtranc(substrc(lcVersion,xx,1),"1234567890.",""))
    lcString = lcString + substrc(lcVersion,xx,1)
  ENDIF
  IF !empty(lcString) AND empty(substrc(lcVersion,xx,1))
    exit
  ENDIF
ENDFOR
IF occurs(".",lcString) > 0
  lcVersion = leftc(lcString,at_c(".",lcVersion)-1)
 ELSE
  lcVersion = lcString
ENDIF
return int(val(lcVersion))




