*
*  X3FWDATA.PRG
*  Create the XX database/xxSYSTEM, xxUSERS, xxSYSID, xxREPCAT
*  tables in the current directory
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  20110
*                      703-369-1759
*  Author:  Drew Speedie
*
*
*  Version 2.0
*
*
*  Usage:
*  do x3fwdata with "VM","ALL"          &&& create VM database, VMSYSTEM.DBF, xxCONFIG.DBF (free table), VMGENPK.DBF, VMUSERS.DBF, VMREPCAT.DBF
*
*  Usage:
*  do x3fwdata with "VM","SYSTEM"       &&& create VM database, VMSYSTEM.DBF, VMLIBCLS.DBF in VM.DBC
*  do x3fwdata with "VM","GENPK"        &&& create VMGENPK.DBF in existing VM.DBC
*  do x3fwdata with "VM","USERS"        &&& create VMUSERS.DBF in existing VM.DBC
*  do x3fwdata with "VM","REPCAT"       &&& create VMREPCAT.DBF in existing VM.DBC
*
lparameters tcParameter1, tcParameter2, tlParameter3
local lcAction, ;
      lcXLibraryDirectory, ;
      lcSystemTable, ;
      llOverwriteExistingX3DBC, ;
      llDontSetDatabaseTo, ;
      lcCurrentSetDatabaseTo
IF type("tcParameter1")#"C" OR empty(tcParameter1)
  wait window "Parameters passed incorrectly to " + program()
  return .f.
ENDIF
lcAction = upper(alltrim(tcParameter1))
lcCurrentSetDatabaseTo = set("DATABASE")
IF lenc(lcAction) # 2
  wait window "Invalid application prefix parameter passed to " + program()
  return .f.
ENDIF
IF type("tcParameter2")#"C" OR empty(tcParameter2)
  wait window "Invalid system table name parameter passed to " + program()
  return .f.
ENDIF
lcSystemTable = upper(alltrim(tcParameter2))
IF !inlist(lcSystemTable,"SYSTEM","GENPK","USERS","REPCAT","ALL")
  wait window "Invalid system table name parameter passed to " + program()
  return .f.
ENDIF
*
*  pass a .T. as the 3rd parameter to leave
*  the current SET DATABASE TO instead of
*  arbitrarily changing it to the 2-character
*  prefix
*
IF type("tlParameter3")#"L"
  llDontSetDatabaseTo = .f.
 ELSE
  llDontSetDatabaseTo = tlParameter3
ENDIF
IF llDontSetDatabaseTo
  lcCurrentSetDatabaseTo = set("DATABASE")
ENDIF
release all except l*
clear windows
clear popups
clear menus
clear dlls
close all
IF llDontSetDatabaseTo
  open database (lcCurrentSetDatabaseTo)
  set database to (lcCurrentSetDatabaseTo)
ENDIF
set talk off
set safety off
set exclusive on
IF lcSystemTable = "ALL"
  do CreateXXData with lcAction, "SYSTEM", llDontSetDataBaseTo
  do CreateXXData with lcAction, "GENPK",  llDontSetDataBaseTo
  do CreateXXData with lcAction, "USERS",  llDontSetDataBaseTo
  do CreateXXData with lcAction, "REPCAT", llDontSetDataBaseTo
 ELSE
  do CreateXXData with lcAction, lcSystemTable, llDontSetDataBaseTo
ENDIF
return


PROCEDURE CreateXXData
lparameters tcAppPrefix, tcTableName, tlDontSetDatabaseTo
wait window "Creating " + tcAppPrefix + " system tables..." nowait
IF tcTableName = "SYSTEM"
  IF file(tcAppPrefix+".DBC")
    *
    * if we're creating the SYSTEM tables, we need to start over
    *
    delete database (tcAppPrefix+".DBC") deletetables
  ENDIF
  create database (tcAppPrefix)
ENDIF
IF tlDontSetDatabaseTo
 ELSE
  open database (tcAppPrefix)
  set database to (tcAppPrefix)
ENDIF

DO CASE

  CASE tcTableName = "SYSTEM"
    do CreateSystemTables with tcAppPrefix
  CASE tcTableName = "GENPK"
    do CreateGeneratedPKTable with tcAppPrefix
  CASE tcTableName = "USERS"
    do CreateUsersTable with tcAppPrefix
  CASE tcTableName = "REPCAT"
        do CreatereportTable with tcAppPrefix
ENDCASE
return



PROCEDURE CreateSystemTables
lparameters tcAppPrefix
*
* system data
*
create table (tcAppPrefix+"SYSTEM") name APPINFO ;
     (App_Item            C(30), ;
      App_ItemDataType    C(1), ;
      App_ItemValue       C(100),;
      App_ItemDescription C(254))
select APPINFO
wait window alias() nowait
=dbsetprop("APPINFO",                     "TABLE", "Comment", "Application-level setup/configuration information")
=dbsetprop("APPINFO.App_Item",            "FIELD", "Comment", "Application setup/configuration item (Primary Key-no duplicates)")
=dbsetprop("APPINFO.App_ItemDataType",    "FIELD", "Comment", "Data type for App_Item (C,N,L,D,T)")
=dbsetprop("APPINFO.App_ItemValue",       "FIELD", "Comment", "App_Item value")
=dbsetprop("APPINFO.App_ItemDescription", "FIELD", "Comment", "Description of App_Item")
m.App_Item            = "SetPathProd"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "VFP path to be set in application setup -- Production"
insert into APPINFO from memvar
m.App_Item            = "TimeoutSeconds"
m.App_ItemDataType    = "N"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "Number of seconds before timeout"
insert into APPINFO from memvar
m.App_Item            = "AppTimerInterval"
m.App_ItemDataType    = "N"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "Interval for the application timer"
insert into APPINFO from memvar
m.App_Item            = "FreeConfigTable"
m.App_ItemDataType    = "C"
m.App_ItemValue       = tcAppPrefix + "CONFIG.DBF"
m.App_ItemDescription = "Filename of the APPCONFIG free table containing app information (extension is required but doesn't have to be .DBF)"
insert into APPINFO from memvar
*
*  xxCONFIG.DBF is a free table
*
local lcDatabase
lcDatabase = set("DATABASE")
set database to
create table (tcAppPrefix+"CONFIG") ;
     (Ap_Item    C(30), ;
      Ap_ItemDTp C(1), ;
      Ap_ItemVal C(100),;
      Ap_ItemDsc C(254))
*
*  Ap_Item, like APPINFO.App_Info, must contain
*  unique values, as if it's a PK/CK
*      
select (tcAppPrefix+"CONFIG")
wait window alias() nowait
m.Ap_Item     = "DevModeExpr"
m.Ap_ItemDTp  = "E"
m.Ap_ItemVal  = [getenv("FPDEV")="ON"]
m.Ap_ItemDsc  = "Expression indicating whether or not app is running in development"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "SetProcedure"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = space(0)
m.Ap_ItemDsc  = "List of procedure files to SET PROCEDURE TO"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "SetClasslib"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "X3FWPPOP.VCX" 
m.Ap_ItemDsc  = "List of class library .VCX files to SET CLASSLIB TO"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "SetLibrary"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = space(0)
m.Ap_ItemDsc  = "List of API library files to SET LIBRARY TO"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "SetPathDev"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = space(0)
m.Ap_ItemDsc  = "VFP path to be set in application setup -- Development"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "UserLoginFormClass"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "frmLogin,X3FW.VCX"
m.Ap_ItemDsc  = "Optional way to set User Login form class/.VCX"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "DevToolbarClass"
m.Ap_ItemDTp  = "C"
IF file("X3TOOLS.VCX")
  m.Ap_ItemVal = "tbrDevTools,X3TOOLS.VCX"
 ELSE
  m.Ap_ItemVal = space(0)
ENDIF
m.Ap_ItemDsc  = "Class name for any desired Developers Toolbar"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "DevToolbarPad"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "Tools"
m.Ap_ItemDsc  = "Pad prompt of system menu popup to which to add Dev toolbar"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "LoginControl"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "AUTO"
m.Ap_ItemDsc  = "Type of auto-login control desired"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "AutoLoginUserID"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "DEFAULT"
m.Ap_ItemDsc  = "User ID to use for auto-login"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "AutoLoginPassword"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "PASSWORD"
m.Ap_ItemDsc  = "Password to use for auto-login"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "TimeoutType"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "N"
m.Ap_ItemDsc  = [Timeout type "A"pp or "M"enu or "N"one]
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "IconFile"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = space(0)
m.Ap_ItemDsc  = ".ICO file for the application"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item    = "ScreenWallpaperFile"
m.Ap_ItemDTp = "C"
m.Ap_ItemVal = space(0)
m.Ap_ItemDsc  = ".BMP file for the use as _Screen wallpaper"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item    = "NoOnErrorDuringDev" 
m.Ap_ItemDTp = "L"
m.Ap_ItemVal = ".T."
m.Ap_ItemDsc  = "Indicates whether ON ERROR error handler is in effect during development"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "Title"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = tcAppPrefix + " Application"
m.Ap_ItemDsc  = "Title string for use in messaging and the title bar"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "Version#"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "1.00"
m.Ap_ItemDsc  = "Application version number"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "Serial#"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "ABC123"
m.Ap_ItemDsc  = "Application serial number" 
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "Copyright"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "Copyright " + right(str(year(date())),4) + " ...  All Rights Reserved."
m.Ap_ItemDsc  = "Copyright information"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "RegisteredToName"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "John Doe"
m.Ap_ItemDsc  = "Name of registered owner"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "RegisteredToCompany"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "John Doe's Company"
m.Ap_ItemDsc  = "Name of registered company/owner"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "ErrorLogDirectory"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = space(0)
m.Ap_ItemDsc  = "Directory (in the path) where ERRORLOG.DBF should be created/maintained"
insert into (tcAppPrefix + "CONFIG") from memvar
m.Ap_Item     = "DefaultReportClass"
m.Ap_ItemDTp  = "C"
m.Ap_ItemVal  = "frmReport,X3FWMISC.VCX"
m.Ap_ItemDsc  = "[ClassName,.VCXFileName] for the default report object class"
insert into (tcAppPrefix + "CONFIG") from memvar
set database to (lcDatabase) 
return



PROCEDURE CreateGeneratedPKTable
*
* system-generated PK data
*
lparameters tcAP
create table (tcAP+"GENPK") name "GENERATEPK" ;
     (GPK_PK            C(40), ;
      GPK_CurrentNumber I(4), ;
      GPK_CurrentAlpha  C(5))
alter table GENERATEPK add primary key GPK_PK tag GPK_PK
index on deleted() tag GPK_Del
=dbsetprop("GENERATEPK",                   "TABLE","Comment","System-generated Primary Keys")
=dbsetprop("GENERATEPK.GPK_PK",            "FIELD","Caption","Primary Key")
=dbsetprop("GENERATEPK.GPK_PK",            "FIELD","Comment","TableName for which to generate PKs")
=dbsetprop("GENERATEPK.GPK_CurrentNumber", "FIELD","Caption","Current PK")
=dbsetprop("GENERATEPK.GPK_CurrentNumber", "FIELD","Comment","Current PK Number in use")
=dbsetprop("GENERATEPK.GPK_CurrentAlpha",  "FIELD","Caption","Current AlphaNumeric PK")
=dbsetprop("GENERATEPK.GPK_CurrentAlpha",  "FIELD","Comment","Current AlphaNumeric PK in use")
return



PROCEDURE CreateUsersTable
lparameters tcAppPrefix
create table (tcAppPrefix+"USERS") name USERS ;
     (Usr_PK                 C(5), ;
      Usr_LanID              C(8), ;
      Usr_Password           C(8) default "PASSWORD", ;
      Usr_Rights             C(1) default "G", ;
      Usr_FirstName          C(15), ;
      Usr_MiddleInitial      C(1), ;
      Usr_LastName           C(15), ;
      Usr_Phone              C(12), ;
      Usr_Fax                C(12), ;
      Usr_Notes              M)
alter table USERS add primary key Usr_PK tag Usr_PK
select USERS
index on upper(Usr_LastName) + upper(Usr_FirstName) + upper(Usr_MiddleInitial) tag Usr_Name
index on upper(Usr_LanID) + Usr_PassWord tag Usr_IDPass
index on Usr_LanID tag Usr_LanID candidate for !deleted()
index on recno() tag Usr_Recno
index on deleted() tag Usr_Del
=dbsetprop("USERS",                       "TABLE","Comment","Users")
=dbsetprop("USERS.Usr_PK",                "FIELD","Caption","Primary Key")
=dbsetprop("USERS.Usr_PK",                "FIELD","Comment","System-generated Primary Key")
=dbsetprop("USERS.Usr_LanID",             "FIELD","Caption","LAN ID")
=dbsetprop("USERS.Usr_LanID",             "FIELD","Comment","User's LAN ID")
=dbsetprop("USERS.Usr_Password",          "FIELD","Caption","Password")
=dbsetprop("USERS.Usr_Password",          "FIELD","Comment","User's Password")
=dbsetprop("USERS.Usr_Rights",            "FIELD","Caption","Rights")
=dbsetprop("USERS.Usr_Rights",            "FIELD","Comment",[Rights level:  "S"upervisor, "G"lobal (non-Supervisor) read-write, "R"ead-Only])
=dbsetprop("USERS.Usr_FirstName",         "FIELD","Caption","First Name")
=dbsetprop("USERS.Usr_MiddleInitial",     "FIELD","Caption","Middle Initial")
=dbsetprop("USERS.Usr_LastName",          "FIELD","Caption","Last Name")
=dbsetprop("USERS.Usr_Phone",             "FIELD","Caption","Phone Number")
=dbsetprop("USERS.Usr_Fax",               "FIELD","Caption","Fax Number")
=dbsetprop("USERS.Usr_Notes",             "FIELD","Caption","Notes")
alter table USERS ;
     alter column Usr_Rights ;
       set check !empty(Usr_Rights) AND inlist(Usr_Rights,"S","G","R") ;
       error [User Rights must be specified: "S"upervisor, "G"lobal, or "R"ead-Only] 


select USERS
wait window alias() nowait
insert into USERS (Usr_PK, ;
                   Usr_Rights, ;
                   Usr_FirstName, ;
                   Usr_MiddleInitial, ;
                   Usr_LastName, ;
                   Usr_LanID) ;
                   values ;
                  ("DEFA", ;
                   "S", ;
                   "Dudley", ;
                   "E", ;
                   "Fault", ;
                   "DEFAULT")
insert into USERS (Usr_PK, ;
                   Usr_Rights, ;
                   Usr_FirstName, ;
                   Usr_MiddleInitial, ;
                   Usr_LastName, ;
                   Usr_LanID) ;
                   values ;
                  ("EXTR", ;
                   "R", ;
                   "External", ;
                   "X", ;
                   "Ternal", ;
                   "EXTERNAL")

local lcDatabasePlusTable
lcDatabasePlusTable = cursorgetprop("DATABASE","USERS") + "!USERS"
CREATE SQL VIEW V_USERS AS ;
    SELECT * ;
    FROM &lcDatabasePlusTable ;
    WHERE USERS.Usr_pk = ?lcUsr_PK
=DBSetProp( 'V_USERS', 'View', 'UpdateType', 1 )
=DBSetProp( 'V_USERS', 'View', 'WhereType', 3 )
=DBSetProp( 'V_USERS', 'View', 'FetchMemo', .T. )
=DBSetProp( 'V_USERS', 'View', 'SendUpdates', .T. )
=DBSetProp( 'V_USERS', 'View', 'UseMemoSize', 255 )
=DBSetProp( 'V_USERS', 'View', 'FetchSize', 100 )
=DBSetProp( 'V_USERS', 'View', 'MaxRecords', -1 )
=DBSetProp( 'V_USERS', 'View', 'Tables', lcDatabasePlusTable )
=DBSetProp( 'V_USERS.usr_pk', 'Field', 'KeyField', .T. )
=DBSetProp( 'V_USERS.usr_pk', 'Field', 'Updatable', .T. )
=DBSetProp( 'V_USERS.usr_pk', 'Field', 'UpdateName', lcDatabasePlusTable+".Usr_pk" )

create table (tcAppPrefix+"USRPRF") name USERPREFS ;
     (Prf_UsrFK            C(5)  , ;
      Prf_Item             C(30) , ;  
      Prf_ItemDataType     C(1)  , ;
      Prf_ItemValue        C(100) NULL, ;
      Prf_ItemDescription  C(254))
select USERPREFS
alter table USERPREFS add primary key Prf_UsrFK + upper(Prf_Item) tag Prf_PK
index on deleted() tag Prf_Del
=dbsetprop("USERPREFS",                     "TABLE", "Comment", "User Preferences")
=dbsetprop("USERPREFS.Prf_UsrFK",           "FIELD", "Comment", "Foreign key to the USERS table")
=dbsetprop("USERPREFS.Prf_Item",            "FIELD", "Comment", "Preference item")
=dbsetprop("USERPREFS.Prf_ItemDataType",    "FIELD", "Comment", "Data type for Prf_ItemValue (C,N,L,D,T)")
=dbsetprop("USERPREFS.Prf_ItemValue",       "FIELD", "Comment", "Prf_Item value")
=dbsetprop("USERPREFS.Prf_ItemDescription", "FIELD", "Comment", "Description of Prf_Item")
alter table USERPREFS add foreign key tag Prf_UsrFK references USERS
select USERS
SCAN
  insert into USERPREFS ;
      (Prf_UsrFK, Prf_Item, Prf_ItemDataType, Prf_ItemValue) ;
      values ;
      (USERS.Usr_PK, "ScreenWindowState", "N", "2")
  insert into USERPREFS ;
      (Prf_UsrFK, Prf_Item, Prf_ItemDataType, Prf_ItemValue) ;
      values ;
      (USERS.Usr_PK, "DevTbrDockPosition", "N", "-1")
ENDSCAN

return


PROCEDURE CreateReportTable
lparameters tcAppPrefix
create table (tcAppPrefix+"REPCAT") name REPORTCATALOG ;
        (Rep_PK          C(5),;
         Rep_Name        C(60), ;
         Rep_Description M(4), ;
         Rep_FRX         C(25), ;
         Rep_Class       C(40), ;
         Rep_Type        C(1))

alter table REPORTCATALOG add primary key Rep_PK tag Rep_PK
INDEX ON UPPER(Rep_Name) tag Rep_Name
index on upper(Rep_FRX) tag Rep_FRX

= DBSETPROP( "REPORTCATALOG.Rep_PK",          "Field", "Caption", "Primary Key" )
= DBSETPROP( "REPORTCATALOG.Rep_FRX",         "Field", "Caption", ".FRX File Name" )
= DBSETPROP( "REPORTCATALOG.Rep_Class",       "Field", "Caption", "Class")
= DBSETPROP( "REPORTCATALOG.Rep_Class",       "Field", "Comment", "Contains [ClassName,VCXFile] report object for this report")
= DBSETPROP( "REPORTCATALOG.Rep_Name",        "Field", "Caption", "Name" )
= DBSETPROP( "REPORTCATALOG.Rep_Description", "Field", "Caption", "Description" )
= DBSETPROP( "REPORTCATALOG.Rep_Type",        "Field", "Caption", "Type" )

return
