*
*  X3FWMAIN.PRG
*  Main calling program for Visual MaxFrame (VM) and
*  Visual MaxFrame professional (VMP) applications 
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  20110
*                      703-369-1759
*  Author:  Drew Speedie
*           With help, input, ideas, bug reports, feature suggestions
*           from:
*             Tom DeMay
*             Ed Lennon
*             Valarie Rollins
*             The Fox Community, especially those who've contributed
*               specific ideas or submitted bug reports (not to
*               mention encouragement), including:
*               Kelly Conway
*               Steven Black
*               Jim Condon
*               Mike Yearwood
*               Dan Wellisch
*               Chaim Caron
*               Antonio Lozada
*
*
*
**************************
*  Version 2.0  10/27/96
**************************
*
*
*
*  COPYRIGHT NOTICE -- Visual MaxFrame
*  ======================================================  
*  All source code and documentation contained in X3FWMAIN and the 
*  Visual MaxFrame (VM) "application framework" were developed at 
*  MaxTech, Inc. and have been placed into the public domain.  You may
*  use, modify, copy, distribute, and demonstrate any source code, 
*  example programs, or documentation contained in X3FWMAIN and the VM 
*  "application framework" supporting files freely without copyright
*  protection.  All files contained in X3FWMAIN and the VM 
*  "application framework" supporting files are distributed in the hope
*  that they will be useful, but WITHOUT ANY WARRANTY; without even the
*  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
*  PURPOSE.  In no event shall its authors, contributors, or
*  distributors be liable for any damages.  The Visual MaxFrame (VM)
*  files are as follows, and may change without notice:
*    X3FW.VCX
*    X3FW.VCT
*    X3FWLIBS.VCX
*    X3FWLIBS.VCT
*    X3FWPPOP.VCX
*    X3FWPPOP.VCT
*    X3FWMAIN.MNX
*    X3FWMAIN.MNT
*    X3FWMAIN.PRG
*    X3FWDATA.PRG
*    X3CNVCHR.PRG
*    X3CPEDIG.PRG
*    X3ERROR.PRG
*    X3GENPK.PRG
*    X3GETVER.PRG
*    X3SETCLS.PRG
*    X3SETHOK.PRG
*    X3WINMSG.PRG
*    X3WRPFRM.PRG
*    X2JSTFIL.PRG
*    X2SETLIB.PRG
*    X2WEXIST.PRG
*
*
*  COPYRIGHT NOTICE -- Visual MaxFrame Professional
*  ======================================================  
*  Copyright 1996 MaxTech.  All Rights Reserved
*
*  
*  REQUIRED ELEMENTS
*  ======================================================  
*  VFP version 3.0b or above is required.  VM and VMP include calls
*  to the PEMSTATUS() function added to version 3.0b, so version
*  3.0 is not supported.
*
*  X3FW.VCX, containing the class definitions of the global
*  Visual MaxFrame application-level objects.  
*
*  The location of the Visual MaxFrame/Professional class libraries
*  must be in the FoxPro path when X3FWMAIN.PRG executes.
*  The location of any application-specific .VCXs that subclass
*  the global Visual MaxFrame application-level objects 
*  must be in the FoxPro path when X3FWMAIN.PRG executes.
*  You have several options:
*      1- SET the PATH in CONFIG.FPW
*      2- execute a SET PATH in xxMAIN.PRG
*      3- make sure the .VCX(s) containing the oLib and oApp 
*         class definitions is in the path when X3FWMAIN.PRG
*         is called, populate the appropriate path-specific
*         records in APPCONFIG and APPINFO
*  
*  While not actually required, we expect the following lines in
*  the CONFIG.FPW in VFP 3.0b applications:
*    TITLE =
*    SYSMENU = OFF
*  And, in VFP 5.0b applications, while not actually required, 
*  Visual MaxFrame/Professional expects the following line in
*  the CONFIG.FPW:
*    SCREEN = OFF
*         
*  For more information on required elements, features, etc., please
*  refer to the accompanying VM.DOC (and VMP.DOC, if you have
*  purchased Visual MaxFrame Professional)
*  
*
*  USAGE 
*  ======================================================  
*    See the example at the end of this .PRG 
*
*
*  PARAMETERS 
*  ======================================================  
*  lParameters
*    tcMainDatabase (O) The name of the main application database, the
*                         one containing the application info table.
*                         (The MaxTech default is to have an xx.DBC
*                         where the "xx" is the 2-char application
*                         prefix, so the main database name is
*                         identical to/synonymous with the main
*                         application database name).  May be the full
*                         drive+path+filename, but be careful -- you'll
*                         likely want this .DBC (and likely its tables)
*                         available in the path.
*                       Defaults to space(0), in which case be sure
*                         to set icMainDatabase property in your
*                         application-object subclass. 
*     tcAppLibClass (O) The name of the class instantiated as oLib 
*                         (plus optional .VCX filename for the class
*                          definition, separated by a comma)
*                       Then add a "|" pipe character delimiter and
*                         as many comma-delimited .VCX names as contain
*                         classes you want instantiated as members of
*                         oLib
*                       Defaults to "cusAppLibs,X3FW|X3FWLIBS"
*                       If you pass this parameter, you MUST include
*                         at least one .VCX filename after the pipe
*                         character or oLib will not be created because
*                         it will have no members.
*                       The class library specified here is referred to
*                         only by indirection during setup, so you'll
*                         want to put an explicit/dummy reference in
*                         your main calling .PRG so the Project Manager
*                         will pull in the .VCX.
*        tcAppClass (O) The name of the class instantiated as oApp
*                       You can optionally add a comma delimiter and
*                         a .VCX name containing the class definition
*                       Defaults to "cusApp,X3FW"
*       tcUserClass (O) The name of the class instantiated as oUser
*                       You can optionally add a comma delimiter and
*                         a .VCX name containing the class definition
*                       Defaults to "cusUser,X3FW"
*      tcFormsClass (O) The name of the class instantiated as oForms
*                       You can optionally add a comma delimiter and
*                         a .VCX name containing the class definition
*                       Defaults to "cusForms,X3FW"
*   tcToolbarsClass (O) The name of the class instantiated as oToolbars
*                       You can optionally add a comma delimiter and
*                         a .VCX name containing the class definition
*                       Defaults to "cusToolbars,X3FW"
*       tcMenuClass (O) The name of the class instantiated as oMenu
*                       You can optionally add a comma delimiter and
*                         a .VCX name containing the class definition
*                       Defaults to "cusMenu,X3FW"
*          tcSplash (0) The string to be used to instantiate a splash
*                       screen.  May be any one of the following:
*                         <.BMP FileName>
*                         <.SCX FileName>
*                         <ClassName,.VCX FileName>
*                       Any passed splash screen is Release()d in
*                       oApp.Init().  Defaults to no splash screen.
*
lparameters tcMainDatabase, ;
            tcAppLibClass, ;
            tcAppClass, ;
            tcUserClass, ;
            tcFormsClass, ;
            tcToolbarsClass, ;
            tcMenuClass, ;
            tcSplash

* 
*  in VFP 5.0, we're expecting that you've set
*    SCREEN = OFF
*  in your production CONFIG.FPW, so to duplicate
*  the effect in development, we'll make the screen
*  invisible now if it isn't already
*
IF X3GETVER() > 3 AND _Screen.Visible 
  _Screen.Visible = .f.
ENDIF

*
*  Install the global error handler to be in effect 
*  from now until oApp.InstallGlobalErrorHandler().
*
*  Note that this initial error handler (local procedure
*  in this program) isn't installed if you've already
*  set an ON ERROR routine in your calling program.
*  This allows full flexibility.
*
*  The default behavior is to install this error handler,
*  then X3ERROR.PRG as specified in oApp.InstallGlobalErrorHandler().
*  But you can override either, by setting your ON ERROR
*  just before calling this program, and by subclassing
*  X3FW.VCX/ctrApp.InstallGlobalErrorHandler().
*
*  Between now and the installation of the global error
*  handler, errors result in a simple MESSAGEBOX(), but
*  it has information regarding the error, even when 
*  running in production.  We did this because we don't
*  formally log errors unti the global error handler
*  is installed along with oApp, but at least the user
*  can write down the error number, program, and line
*  to help you debug
*
*  During application setup (between now and READ EVENTS),
*  if an (anticipated) trapped error occurs, we handle it 
*  and advise the user via a MESSAGEBOX() that includes
*  the pcInstallationError string containing the general
*  location of the error.
*
IF empty(on("ERROR"))
  on error do InitialErrorHandler with error(), ;
                                       program(), ;
                                       lineno() 
ENDIF

*
*  Yes, I know the following isn't "pure OOP" <sigh>,
*  but it's a heckuva a lot easier than any other way
*  I know, so I'm going to do it anyway...  
*
private pcInstantiationError   
pcInstantiationError = space(0)
*
*  pcInstantiationError is set throughout the instantiation process
*  of each of the application-level objects


set talk off
IF version(2) = 0
  *
  *  runtime
  *
  IF empty(_Screen.Icon)
    *
    *  if the calling program hasn't set an explicit
    *  _Screen.Icon, suppress the VFP FoxHead icon
    *  until any application-icon is installed by oApp
    *
    _Screen.Icon = space(0)
  ENDIF
 ELSE
  *
  *  from the Command Window
  *
  activate screen
  clear
ENDIF
IF type("tcMainDatabase") = "C" ;
     AND !empty(tcMainDatabase) 
  tcMainDatabase = upper(alltrim(tcMainDatabase)) 
 ELSE
  pcInstantiationError = program() + " tcMainDatabase parameter not passed"
  do TroubleGettingIn
 return
ENDIF

***************************************************
*  set classlib to X3FWPPOP is required as it's used
*  all over the place in the framework.  By default
*  it's also set in oApp.SetClasslib()
***************************************************
IF !x3setcls("X3FWPPOP.VCX")
  pcInstantiationError = program() + " set classlib to X3FWPPOP.VCX failed"
  do TroubleGettingIn
ENDIF

***************************************************
*  get rid of any VFP toolbars (they'll be
*  restored when this .PRG terminates)
***************************************************
*
*  get rid of any VFP toolbars, DEFINE CLASS cusPushPopVFPToolbars
*  is contained here in X3FWMAIN.PRG
*
_Screen.AddObject("oVFPToolbars","cusPushPopVFPToolbars")
WITH _Screen.oVFPToolbars
  .Top = -1
  .Left = -1
ENDWITH
*local loVFPToolbars
*loVFPToolbars = createobject("cusPushPopVFPToolbars")

***************************************************
*  install any indicated Splash Screen
***************************************************
*
*  we're expecting the following 2 SETtings to be
*  done in the CONFIG.FPW, but just in case:
*
IF set("SYSMENU") # "OFF" 
  set sysmenu off
ENDIF
IF set("STATUS BAR") # "OFF" 
  set status bar off
ENDIF  

IF type("tcSplash") = "C" AND !empty(tcSplash)
  local loSplashScreen
  loSplashScreen = InstallSplashScreen(upper(alltrim(tcSplash)))
  IF type("loSplashScreen") = "O" ;
       AND !isnull(loSplashScreen)
    * splash screen installation was successful
   ELSE
    pcInstantiationError = program() + " problem installing the tcSplash splash screen"
    do TroubleGettingIn
    return
  ENDIF  
ENDIF
wait window space(0) timeout .1   &&& forces the screen update
local llCheck, lcClassDefinition

***************************************************
* create the library object:
***************************************************
IF isnull(tcAppLibClass)
  *
  *  this particular app needs no library object
  *
 ELSE
  local lcLibraryVCXs
  lcLibraryVCXs = space(0)
  IF type("tcAppLibClass") = "C" AND !empty(tcAppLibClass)
    lcClassDefinition = upper(alltrim(tcAppLibClass))
    IF "|" $ lcClassDefinition
      lcLibraryVCXs     = substrc(lcClassDefinition,at_c("|",lcClassDefinition)+1)
      lcClassDefinition = leftc(lcClassDefinition,at_c("|",lcClassDefinition)-1)
     ELSE
      *
      *  DANGER!  If you haven't passed at least one
      *           .VCX name, oLib will be created
      *           but then released because it'll
      *           have no members
      *
    ENDIF
    lcClassDefinition = ParseClassNameAndSetLibrary(lcClassDefinition)
   ELSE 
    *
    *  default/not passed
    *
    lcLibraryVCXs     = "X3FWLIBS"
    lcClassDefinition = ParseClassNameAndSetLibrary("cusAppLibs,X3FW")
  ENDIF
  IF isnull(lcClassDefinition)
    pcInstantiationError = program() + " problem SETting CLASSLIB TO tcAppLibClass" 
    do TroubleGettingIn 
    return
  ENDIF
  oLib = createobject(lcClassDefinition,lcLibraryVCXs)
  release lcLibraryVCXs
  IF type("oLib")="O" AND !isnull(oLib) 
   ELSE
    do TroubleGettingIn
    return
  ENDIF
  IF amembers(junk,oLib,2) = 0
    *
    *  no member library objects --
    *  no need to keep oLib around
    *
    release oLib
  ENDIF
  release junk
ENDIF 

***************************************************
* create the application object:
***************************************************
IF type("tcAppClass") = "C" AND !empty(tcAppClass)
  lcClassDefinition = ParseClassNameAndSetLibrary(tcAppClass)
 ELSE 
  *
  *  default/not passed
  *
  lcClassDefinition = ParseClassNameAndSetLibrary("ctrApp,X3FW")
ENDIF
IF isnull(lcClassDefinition)
  pcInstantiationError = program() + ;
       " problem SETting CLASSLIB TO tcAppClass" 
  do TroubleGettingIn 
  return
ENDIF
oApp = createobject(lcClassDefinition,tcMainDatabase) 
IF type("oApp")="O" AND !isnull(oApp) 
 ELSE
  do TroubleGettingIn 
  return
ENDIF
IF oApp.GetPProp("ilTerminateAfterInit")
  *
  *  during oApp.Init(), a problem was encountered
  *  that was handled there, but indicates the app
  *  should be terminated.  For example, it may have
  *  been determined that the user was attempting to
  *  fire up a 2nd session of the app but 
  *  oApp.ilAllowMultipleInstances = .f.
  *
  do TroubleGettingIn with .t.
  return
ENDIF
*
*  execute any desired AfterInstallApp() code
*
llCheck = oApp.AfterInstallApp()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

*
*  release any splash screen
*
_Screen.Visible = .t.   &&& VFP5.0-specific, irrelevant in VFP3.0b
oApp.ReleaseSplash()

***************************************************
* create the User object:
***************************************************
llCheck = oApp.BeforeInstallUser()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 
IF type("tcUserClass") = "C" AND !empty(tcUserClass)
  lcClassDefinition = ParseClassNameAndSetLibrary(tcUserClass)
 ELSE 
  *
  *  default/not passed
  *
  lcClassDefinition = ParseClassNameAndSetLibrary("cusUser,X3FW")
ENDIF
IF isnull(lcClassDefinition)
  pcInstantiationError = program() + " problem SETting CLASSLIB TO tcUserClass" 
  do TroubleGettingIn 
  return
ENDIF
oUser = createobject(lcClassDefinition) 
IF type("oUser")="O" AND !isnull(oUser) 
 ELSE
  *
  * oUser object not created
  *   or  
  * something went wrong with user login form
  *
  do TroubleGettingIn 
  return
ENDIF
IF eof(oUser.GetPProp("icUsersTableName"))
  *
  *  password login was unsuccessful
  *
  do TroubleGettingIn with .t.
  return
ENDIF
*
*  execute any desired AfterInstallUserMgr() code
*
llCheck = oApp.AfterInstallUser()
IF IsThereAnyTrouble(llCheck)
  do ReleaseAppObjects
  return
ENDIF 

***************************************************
* create the Forms manager object:
***************************************************
llCheck = oApp.BeforeInstallFormsMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 
IF type("tcFormsClass") = "C" AND !empty(tcFormsClass)
  lcClassDefinition = ParseClassNameAndSetLibrary(tcFormsClass)
 ELSE
  *
  *  default/not passed
  *
  lcClassDefinition = ParseClassNameAndSetLibrary("cusForms,X3FW")
ENDIF
IF isnull(lcClassDefinition)
  pcInstantiationError = program() + " problem SETting CLASSLIB TO tcFormsClass" 
  do TroubleGettingIn 
  return
ENDIF
oForms = createobject(lcClassDefinition)
IF type("oForms")="O" AND !isnull(oForms) 
 ELSE
  do TroubleGettingIn 
  return
ENDIF
*
*  execute any desired AfterInstallFormsMgr() code
*
llCheck = oApp.AfterInstallFormsMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

***************************************************
* create the toolbar manager object:
***************************************************
llCheck = oApp.BeforeInstallToolbarsMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 
IF type("tcToolbarsClass") = "C" AND !empty(tcToolbarsClass)
  lcClassDefinition = ParseClassNameAndSetLibrary(tcToolbarsClass)
 ELSE 
  *
  *  default/not passed
  *
  lcClassDefinition = ParseClassNameAndSetLibrary("cusToolbars,X3FW")
ENDIF
IF isnull(lcClassDefinition)
  pcInstantiationError = program() + " problem SETting CLASSLIB TO tcToolbarsClass" 
  do TroubleGettingIn  
  return
ENDIF
oToolbars = createobject(lcClassDefinition)
IF type("oToolbars")="O" AND !isnull(oToolbars) 
 ELSE
  do TroubleGettingIn  
  return
ENDIF
*
*  execute any desired AfterInstallToolbarsMgr() code
*
llCheck = oApp.AfterInstallToolbarsMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

***************************************************
* create the Menu object
***************************************************
llCheck = oApp.BeforeInstallMenuMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 
IF type("tcMenuClass") = "C" AND !empty(tcMenuClass)
  lcClassDefinition = ParseClassNameAndSetLibrary(tcMenuClass)
 ELSE 
  *
  *  default/not passed
  *
  lcClassDefinition = ParseClassNameAndSetLibrary("cusMenu,X3FW")
ENDIF
IF isnull(lcClassDefinition)
  pcInstantiationError = program() + " problem SETting CLASSLIB TO tcMenuClass" 
  do TroubleGettingIn 
  return
ENDIF
oMenu = createobject(lcClassDefinition) 
IF type("oMenu")="O" AND !isnull(oMenu) 
 ELSE
  do TroubleGettingIn 
  return
ENDIF
*
*  execute any desired AfterInstallMenu() code
*
llCheck = oApp.AfterInstallMenuMgr()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

***************************************************
*  release the local memvars used here for class instantiation 
***************************************************
release llClassProblem
release lcClassDefinition

***************************************************
*  install any specified bitmap wallpaper:
***************************************************
*
*  execute any desired InstallBitmapWallpaper() code
*
llCheck = oApp.InstallBitmapWallpaper()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

***************************************************
*  application wait state 
***************************************************
*
*  execute final setup code to execute just
*  before READ EVENTS 
*
llCheck = oApp.BeforeReadEvents()
IF IsThereAnyTrouble(llCheck) 
  do ReleaseAppObjects
  return
ENDIF 

release llCheck, pcInstantiationError

oApp.ReadEvents()

***************************************************
*  application cleanup
***************************************************
do ReleaseAppObjects

return



PROCEDURE ReleaseAppObjects
*
*  release the objects created here in the reverse order
*  in which they were instantiated
*
*  each contains cleanup code in Destroy() 
*
activate screen
clear
release oMenu
release oToolbars
release oForms 
release oUser
release oLib
release oApp
IF type("_Screen.oVFPToolbars") = "O" ;
     AND !isnull(_Screen.oVFPToolbars)
  _Screen.RemoveObject("oVFPToolbars")
ENDIF
*
*  these are likely needed if the app has to be 
*  prematurely aborted in our IsThereAnyTrouble
*  procedure
*
close all
set library to
clear all
on error
_Screen.Visible = .t.
return



PROCEDURE TroubleGettingIn
*
*  hit the road 
*
lparameters tlNoMessage, tlNoRelease
IF !tlNoMessage
  lcMessage = "Unable to set up the system.  " + ;
              "Please report this problem, along with the following information:" + chr(13) + chr(13) + ;
              lower(pcInstantiationError)
  ?? chr(7)
  =messagebox(lcMessage,0+16+0,"Please report this problem")
ENDIF
IF !tlNoRelease
  do ReleaseAppObjects
ENDIF
on error
return



PROCEDURE IsThereAnyTrouble
*
*  Check for trouble on executing the 
*  oApp.Before/After...() methods
*
*  NOTE:  the oApp.Before/After...() method needs to
*         return one of 3 values:
*         .NULL. = bail out of the app, 
*                  no additional message to the user
*            .F. = abort the app, give the user
*                  the standard TroubleGettingIn()
*                  message
*            .T. = everything's AOK
*
lparameters tlSuccessIndicator 
local llWeveGotTrouble
llWeveGotTrouble = isnull(tlSuccessIndicator) OR !tlSuccessIndicator
DO CASE
  CASE isnull(tlSuccessIndicator)
    * bail out of the app, no additional
    * message to the user
    do TroubleGettingIn with .t.,.t.
  CASE !tlSuccessIndicator
    * problem -- alert the user
    do TroubleGettingIn with .f.,.t.
  OTHERWISE tlSuccessIndicator
    * success!
ENDCASE 
return llWeveGotTrouble




PROCEDURE ParseClassNameAndSetLibrary
*
*  take action based on the passed tcString
*  and pass back the indicated Class Name
*  
*  lParameters
*    tcString     (R) String to be parsed
*
lparameters tcString
IF type("tcString") = "C" AND !empty(tcString)
  local lcString
  lcString = upper(alltrim(tcString))
 ELSE
  return space(0)
ENDIF
local lcClassName
IF "," $ lcString
  lcClassName = leftc(lcString,at_c(",",lcString)-1)
  *
  *  also SET CLASSLIB TO whatever is after the comma
  *
  local lcClassLib
  lcClassLib = substrc(lcString,at_c(",",lcString)+1)
  IF !empty(lcClassLib)
    local lcOnError
    lcOnError = on("ERROR")
    on error lcClassName = .NULL.
    set classlib to (lcClassLib) additive
    on error &lcOnError
  ENDIF
 ELSE
  lcClassName = lcString
ENDIF
return lcClassName


PROCEDURE InstallSplashScreen
*
*  install the indicated splash screen
*
*  NOTE: the splash screen is Release()d
*        in oApp.Init()
*
*
*  lParameters
*    tcSplashScreen (R) Splash screen to instantiate
*                       May be any one of the 
*                       following:
*                       <.BMP file name>
*                       <.SCX file name>
*                       <ClassName,.VCX file name>
*
*
lparameters tcSplashScreen
local loScreen
loScreen = .f.
DO CASE
  ***********************************
  CASE ".BMP" $ tcSplashScreen
  ***********************************
    *
    *  only a .BMP filename was passed -- use the
    *  default splash screen in the X3FW library
    *
    *  NOTE:  the following explicit SET CLASSLIB TO
    *         pulls the X3FW.VCX into the project
    *
    set classlib to x3fw additive
    loScreen = createobject("frmSplash")
    IF isnull(loScreen)
     ELSE
      loScreen.BackColor = _Screen.BackColor
      loScreen.imgSplash.Picture = tcSplashScreen
      loScreen.imgSplash.Top = 0
      loScreen.imgSplash.Left = 0
      loScreen.Height = loScreen.imgSplash.Height
      loScreen.Width = loScreen.imgSplash.Width
      loScreen.AutoCenter = .t.
      loScreen.Show()
    ENDIF
  ***********************************
  CASE ".SCX" $ tcSplashScreen
  ***********************************
    *
    *  an .SCX filename was passed -- DO it
    *
    IF file(tcSplashScreen)
      do form (tcSplashScreen) name loScreen
    ENDIF
  ***********************************
  CASE "," $ tcSplashScreen
  ***********************************
    *
    *  a "class,vcxfilename" was passed, create
    *  an instance of it
    *
    local lcClassName
    lcClassName = ParseClassNameAndSetLibrary(tcSplashScreen)
    IF isnull(lcClassName)
      * bad .VCX class library
     ELSE
      loScreen = createobject(lcClassName)
      loScreen.Show()
    ENDIF
ENDCASE
IF type("loScreen") = "O" AND !isnull(loScreen)
  loScreen.Tag = "ThisIsAnX3FWMAINSplashScreen"
ENDIF
return loScreen



PROCEDURE InitialErrorHandler
*
*  simple error handler intended for use from the
*  beginning of X3FWMAIN.PRG execution until the
*  "real" global error handler is set during oApp
*  setup
*
lParameters tnError, tcProgram, tnLine 
pcInstantiationError = pcInstantiationError + chr(13) + ;
                       space(2) + "Program:  " + lower(tcProgram) + chr(13) + ;
                       space(4) + "Error #" + alltrim(str(tnError)) + chr(13) + ;
                       space(4) + "Line #" + alltrim(str(tnLine)) 
do TroubleGettingIn 
do ReleaseAppObjects
_Screen.Visible = .t.
quit
return


PROCEDURE pmFakeOut
*
*  this procedure is never called, only serves
*  to pull files into the Project Manager that
*  might only be called by macro substitution
*  or indirection
*
set classlib to x3fwlibs.vcx
set classlib to x3fwppop.vcx
return




DEFINE CLASS cusPushPopVFPToolbars AS Custom
Height = 1
Width = 1
protected iaVFPToolbars[1]
PROCEDURE Init
  dimension THIS.iaVFPToolbars[11,2]
  THIS.iaVFPToolbars[1,1] = "Form Designer"
  THIS.iaVFPToolbars[2,1] = "Standard"
  THIS.iaVFPToolbars[3,1] = "Layout"
  THIS.iaVFPToolbars[4,1] = "Query Designer"
  THIS.iaVFPToolbars[5,1] = "View Designer"
  THIS.iaVFPToolbars[6,1] = "Color Palette"
  THIS.iaVFPToolbars[7,1] = "Form Controls"
  THIS.iaVFPToolbars[8,1] = "Database Designer"
  THIS.iaVFPToolbars[9,1] = "Report Designer"
  THIS.iaVFPToolbars[10,1] = "Report Controls"
  THIS.iaVFPToolbars[11,1] = "Print Preview"
  local lnCount
  FOR lnCount = 1 to alen(THIS.iaVFPToolbars,1)
  THIS.iaVFPToolbars[lnCount,2] = wvisible(THIS.iaVFPToolbars[lnCount,1])
    IF THIS.iaVFPToolbars[lnCount,2]
      hide window (THIS.iaVFPToolbars[lnCount,1])
    ENDIF
  ENDFOR
ENDPROC
PROCEDURE Destroy
  local lnCount
  FOR lnCount = 1 TO alen(THIS.iaVFPToolbars,1)
    IF THIS.iaVFPToolbars[lnCount,2]
      show window (THIS.iaVFPToolbars[lnCount,1])
    ENDIF
  ENDFOR
ENDPROC
ENDDEFINE










#IF .f.

****************************************************
*
*  The following is an example of the xxMAIN.PRG main
*  calling program for an application using the Visual
*  MaxFrame/Visual MaxFrame Professional application
*  framework for Visual FoxPro.
*
*  If you have Visual MaxFrame Professional, see MTMAIN.PRG
*  of the example application for a more detailed example.
*
****************************************************

*
*  xxMAIN.PRG
*  Example of a main calling program for a standard
*  Visual MaxFrame/Visual MaxFrame Professional 
*  application 
*
clear all
clear all
do x3fwmain with "XX",.f.,"ctrXXApplication"
return


PROCEDURE pmFakeOut
*
*  Since we generally SET CLASSLIB TO, call 
*  .SCX-based forms, menus, etc. via indirection,
*  we place direct calls to them here to force the
*  Project Manager to pull them into the .PJX
*  project.  Note that this procedure is never
*  called in the application.
*
set library to foxtools.fll
return


#ENDIF .f. 











*  disregard the following, included here for
*  administration purposes only


*  
*  The following is a list of enhancements made to
*  Visual MaxFrame/Visual MaxFrame Professional since
*  the last major version change.
*  Retrofitting instructions refer to modifications
*  you must make to existing applications to keep
*  them in synch with the VMP framework.
*
*  For a complete list of all version revisions
*  since v1.0, see the accompanying VMREVHST.DOC
*

#IF .f.     RECENT REVISION HISTORY

*****************************************************
*  Version 1.95 ENHANCEMENTS (Visual MaxFrame)
*    and any revisions required from Version 1.94
*  10/26/96
*****************************************************

none

****************************************************
****************************************************
****************************************************
****************************************************
****************************************************
*
*  VMP!
*
****************************************************
****************************************************
****************************************************
****************************************************
****************************************************


*****************************************************
*  Version 1.95 ENHANCEMENTS (VMP)
*    and any revisions required from Version 1.94
*  10/26/96
*****************************************************

#endif .f.   RECENT REVISION HISTORY

