*
*  X3WINMSG.PRG
*  Call the Visual FoxPro MESSAGEBOX() function 
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  20110
*                      703-369-1759
*  Author:  Drew Speedie
*
*  Returns the upper character string of the button pushed
*
*  NOTE: MESSAGEBOX() WILL NOT TIMEOUT!! 
*
*
*  TYPICAL USAGE: 
*    do x3winmsg with "This record is currently being updated by ;
*                      another user.  Please try again later"
*
*    IF x3winmsg("Are you SURE you want to delete this record?",;
*         .f.,"YN","?") = "YES"
*      * proceed with the deletion
*    ENDIF
*
*
*  PARAMETERS: 
*    tcMessage   (REQ) message to be displayed
*      tcTitle   (OPT) title on the message window
*                Defaults to "Please Note" (can be passed as "" for no title)
*    tcButtons   (OPT) string yielding the desired command buttons
*                "OC"      OK/Cancel
*                "ARI"     Abort/Retry/Ignore
*                "YNC"     Yes/No/Cancel
*                "YN"      Yes/No
*                "RC"      Retry/Cancel
*                Defaults to "OK" 
*       tcIcon   (OPT) icon to be displayed
*                "S"       Stop sign
*                "?"       Question mark
*                "!"       Exclamation point
*                "I"       Information icon
*                Defaults to "!" (can be passed as "" for no icon)
*                NOTE:  According to The Windows Interface
*                       Guidelines for Software Design, the "?"
*                       icon is no longer recommended
*     tlSilent   (OPT) silent/don't ring the bell
*                Defaults to .t. -- bell rings
*
parameters tcMessage, ;
           tcTitle, ;
           tcButtons, ;
           tcIcon, ;
           tlSilent

IF type("_Screen.ActiveForm.ActiveControl") = "O" ;
     AND !isnull(_Screen.ActiveForm.ActiveControl) ;
     AND type("_Screen.ActiveForm.LockScreen") = "L" ;
     AND _Screen.ActiveForm.LockScreen ;
     AND upper(_Screen.ActiveForm.ActiveControl.BaseClass) = "GRID"
  *
  *  This code is here because of the code we have
  *  in X3FW.VCX/grdBase.BeforeRowColChange() to
  *  fix the ugly "back-scrolling" when the columns'
  *  gross width exactly match the grid width and
  *  the user back-tabs from the first column.
  *  Unfortunately, =MESSAGEBOX() fires the 
  *  Grid.BeforeRowColChange() but not the Grid.Valid(),
  *  so we put this code here to make sure the
  *  active form.LockScreen doesn't remain locked
  *  after presenting a MESSAGEBOX() while in a
  *  grid
  *
  _Screen.ActiveForm.LockScreen = .f.
ENDIF  

local lcRetVal, lnButtons, lnFocus, lnIcon
tcMessage = alltrim(tcMessage)
IF type("tcTitle") = "C"
  tcTitle = alltrim(tcTitle)
 ELSE
  tcTitle = "Please Note"
ENDIF
lnFocus   = 0
IF type("tcButtons") # "C"
  tcButtons = "O"
  lnButtons = 0                &&& OK                  {ESCAPE} does nothing
 ELSE
  tcButtons = upper(alltrim(tcButtons))
  IF "|" $ tcButtons
    lnFocus = int(val(rightc(tcButtons,1)))
  ENDIF 
  DO CASE 
    CASE tcButtons = "OC"      &&& OK/Cancel           {ESCAPE} yields Cancel
      lnButtons = 1
    CASE tcButtons = "ARI"     &&& Abort/Retry/Ignore  {ESCAPE} does nothing
      lnButtons = 2
    CASE tcButtons = "YNC"     &&& Yes/No/Cancel       {ESCAPE} yields Cancel
      lnButtons = 3
    CASE tcButtons = "YN"      &&& Yes/No              {ESCAPE} does nothing
      lnButtons = 4
    CASE tcButtons = "RC"      &&& Retry/Cancel        {ESCAPE} does nothing
      lnButtons = 5
    OTHERWISE                  &&& default to OK       {ESCAPE} does nothing
      tcButtons = "O"
      lnButtons = 0 
  ENDCASE 
ENDIF
IF lnFocus = 1 OR lnFocus>lenc(tcButtons)
  lnFocus = 0
 ELSE
  lnFocus = IIF(lnFocus=2,256,512)
ENDIF
IF type("tcIcon") # "C"
  lnIcon = 48                  &&& Exclamation point
 ELSE
  DO CASE
    CASE empty(tcIcon)         &&& no icon
      lnIcon = 0
    CASE tcIcon = "S"          &&& Stop sign
      lnIcon = 16
    CASE tcIcon = "?"          &&& Question mark
      lnIcon = 32
    CASE tcIcon = "I"          &&& Information icon
      lnIcon = 64
    OTHERWISE                  &&& Exclamation point
      lnIcon = 48    
  ENDCASE
ENDIF
IF type("tlSilent")#"L" OR !tlSilent
  ?? chr(7)
ENDIF
lxRetVal = messagebox(tcMessage,lnButtons+lnFocus+lnIcon,tcTitle) 
DO CASE
  CASE lxRetVal = 1
    lxRetVal = "OK"
  CASE lxRetVal = 2
    lxRetVal = "CANCEL"
  CASE lxRetVal = 3
    lxRetVal = "ABORT"
  CASE lxRetVal = 4
    lxRetVal = "RETRY"
  CASE lxRetVal = 5
    lxRetVal = "IGNORE"
  CASE lxRetVal = 6
    lxRetVal = "YES"
  CASE lxRetVal = 7
    lxRetVal = "NO"
  OTHERWISE
    lxRetVal = "ESCAPE"
ENDCASE
return lxRetVal
 
 
 
 
PROCEDURE keywords
* MESSAGING|FOXTOOLS|INTERFACE DESIGN
return 